//
//  SynfigApplication.h
//  Synfig Studio
//
//  Created by Robert Quattlebaum on Sun Nov 21 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "X11Application.h"

@interface SynfigApplication : X11Application {

}

@end
