//
//  SynfigApplication.m
//  Synfig Studio
//
//  Created by Robert Quattlebaum on Sun Nov 21 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "SynfigApplication.h"


@implementation SynfigApplication

@end
